#include "func_pars.h"
#include <iostream>
#include <algorithm>
#include <cmath>

template <class T>
function_parser<T>::function_parser()
{
	try
	{
		this->_incognita = const_cast<char *>("x");
		this->_original_func = new std::string("1");
		this->_double_operator = new std::map<char, TDPUNT>();
		this->_one_operator = new std::map<char, TOPUNT>();
		this->_half_map = new std::map<char *, char>();
		this->_assign = 'a';
	}
	catch(std::bad_alloc ex)
	{
		std::cerr << "Exception: " << ex.what() << std::endl;
	}
}
template <class T>
function_parser<T>::function_parser(const char *func, const char *inc)
{
	try
	{
		this->_original_func = new std::string(func);
		this->_incognita = const_cast<char *>(inc);
		this->_double_operator = new std::map<char, TDPUNT>();
		this->_one_operator = new std::map<char, TOPUNT>();
		this->_half_map = new std::map<char *, char>();
		this->_assign = 'a';
	}
	catch(std::bad_alloc ex)
	{
		std::cerr << "Exception: " << ex.what() << std::endl;
	}
}
template <class T>
function_parser<T>::~function_parser()
{
	delete[] this->_incognita;
	delete this->_original_func;
	delete this->_double_operator;
	delete this->_one_operator;
}
template <class T>
char * function_parser<T>::get_func() const
{
	return const_cast<char *>(this->_original_func->c_str());
}
template <class T>
char * function_parser<T>::get_inc() const
{
	return this->_incognita;
}
template <class T>
bool function_parser<T>::add_one_operator(char *name, TOPUNT func)
{
	if(this->_half_map[name] != this->_half_map->end())
	{
		this->_half_map[name] = ++this->_assign;
		this->_one_operator[this->_half_map[name]] = func;
		return true;
	}
	else
		return false;
}
template <class T>
bool function_parser<T>::add_double_operator(char *name, TDPUNT func)
{
	if(this->_half_map[name] != this->_half_map->end())
	{
		this->_half_map[name] = ++this->_assign;
		this->_double_operator[this->_half_map[name]] = func;
		return true;
	}
	else
		return false;
}
template <class T>
bool function_parser<T>::find_errors()
{
	//elimino tutti gli spazi:
	trim(*this->_original_func);
	//da fare
	return true;
}
template <class T>
void function_parser<T>::trim(string &str)
{
	std::string::iterator new_end = std::remove(str.begin(), str.end(), ' ');
	std::string::iterator itr;
	int i;

	for(itr = str.begin(), i = 0; itr != new_end; itr++, i++)
		;
	str.resize(i);
}
template <class T>
void function_parser<T>::load_default_operators()
{
	this->_half_map["+"] = '+';
	this->_double_operator['+'] = function_parser::my_plus;
	this->_half_map["-"] = '-';
	this->_double_operator['-'] = function_parser::my_minus;
	this->_half_map["*"] = '*';
	this->_double_operator['*'] = function_parser::my_mult;
	this->_half_map["/"] = '/';
	this->_double_operator['/'] = function_parser::my_div;
}
template <class T>
void function_parser<T>::load_default_math_operations()
{
	this->_half_map["sin"] = ++this->_assign;
	this->_one_operator[this->_assign] = function_parser::my_sin;
	this->_half_map["cos"] = ++this->_assign;
	this->_one_operator[this->_assign] = function_parser::my_cos;
	this->_half_map["tan"] = ++this->_assign;
	this->_one_operator[this->_assign] = function_parser::my_tan;
	this->_half_map["cotan"] = ++this->_assign;
	this->_one_operator[this->_assign] = function_parser::my_cotan;
}
template <class T>
T function_parser<T>::my_plus(T n1, T n2)
{
	return n1 + n2;
}
template <class T>
T function_parser<T>::my_minus(T n1, T n2)
{
	return n1 - n2;
}
template <class T>
T function_parser<T>::my_mult(T n1, T n2)
{
	return n1 * n2;
}
template <class T>
T function_parser<T>::my_div(T n1, T n2) throw(int)
{
	if(n2 != 0)
		return n1 / n2;
	else
		throw 1;
}
template <class T>
T function_parser<T>::my_sin(T n)
{
	return ::sin(n);
}
template <class T>
T function_parser<T>::my_cos(T n)
{
	return ::cos(n);
}
template <class T>
T function_parser<T>::my_tan(T n)
{
	return ::tan(n);
}
template <class T>
T function_parser<T>::my_cotan(T n)
{
	return 1 / ::tan(n);
}








